using System;
using System.Collections.Generic;
using System.Text;
using System.Windows.Forms;

namespace net1
{
    public class Person
    {
        public Person(string name, int height, int weight, EyeColorEnum eyeColor)
        {
            this.innerName = name;
            this.innerHeight = height;
            this.innerWeight = weight;
            this.innerEyeColor = eyeColor;
        }

        public string GetHeightText()
        {
            return "Wzrost: " + innerHeight.ToString();
        }

        public string GetWeightText()
        {
            return "Waga: " + innerWeight.ToString();
        }

        public string GetEyesText()
        {
            return "Oczy: " + innerEyeColor.ToString();
        }

        private string innerName = "John Doe";
        private int innerHeight = 180;
        private int innerWeight = 90;
        private EyeColorEnum innerEyeColor = EyeColorEnum.Grey;

        public string personName
        {
            get { return innerName; }
            set { innerName = value; }
        }

        public int personHeight
        {
            get { return innerHeight; }
            set { innerHeight = value; }
        }

        public int personWeigth
        {
            get { return innerWeight; }
            set { innerWeight = value; }
        }

        public EyeColorEnum personEyes
        {
            get { return innerEyeColor; }
            set { innerEyeColor = value; }
        }

        public enum EyeColorEnum : short
        {
            Blue,
            Grey,
            Green,
            Brown,
            Undefined
        }
    }

    public class Document
    {
        public Document()
        {
            innerDocName = System.DateTime.Now.ToString();
        }

        public void Register(IDocumentView view) {
            view.FormClosed += new System.Windows.Forms.FormClosedEventHandler(view_FormClosed);
            views.Add(view);
            innerViewCount++;
        }

        void view_FormClosed(object sender, System.Windows.Forms.FormClosedEventArgs e)
        {
            if (views.IndexOf((IDocumentView)sender) < 0)
                throw new Exception("Not found!");
            else
            {
                views.Remove((IDocumentView)sender);
                innerViewCount--;
            }
        }

        private string innerDocName;

        public string docName
        {
            get { return innerDocName; }
        }

        private List<IDocumentView> views = new List<IDocumentView>();
        private int innerViewCount;
        public int viewCount {
            get { return innerViewCount; }
        }

        private List<Person> people = new List<Person>();

        public enum UpdateTask
        {
            Add,
            Delete,
            Modify
        }

        public bool AddElement()
        {
            EditForm editBox = new EditForm();
            editBox.ShowDialog();
            if (editBox.DialogResult == DialogResult.OK)
            {
                Person person = new Person(editBox.personName, editBox.personHeight, editBox.personWeight, editBox.personEyes);
                people.Add(person);
                UpdateViews(person, UpdateTask.Add);
                editBox.Dispose();
                return true;
            }
            return false;
        }

        public Person[] peopleList
        {
            get { return people.ToArray(); }
        }

        public bool DeleteElement(Person person)
        {
            if (people.IndexOf(person) != -1)
            {
                people.Remove(person);
                UpdateViews(person, UpdateTask.Delete);
                return true;
            }
            else
            {
                return false;
            }
        }

        public bool ModifyElement(Person person)
        {
            EditForm editBox = new EditForm();

            editBox.personName = person.personName;
            editBox.personHeight = person.personHeight;
            editBox.personWeight = person.personWeigth;
            editBox.personEyes = person.personEyes;
            editBox.ShowDialog();

            if (editBox.DialogResult == DialogResult.OK)
            {
                person.personName = editBox.personName;
                person.personHeight = editBox.personHeight;
                person.personWeigth = editBox.personWeight;
                person.personEyes = editBox.personEyes;
                UpdateViews(person, UpdateTask.Modify);
                editBox.Dispose();
                return true;
            }
            return false;
        }

        private bool UpdateViews(Person person, UpdateTask task)
        {
            foreach (IDocumentView view in views)
            {
                switch (task)
                {
                    case UpdateTask.Add:
                        view.AddElement(person);
                        break;
                    case UpdateTask.Delete:
                        view.DeleteElement(person);
                        break;
                    case UpdateTask.Modify:
                        view.ModifyElement(person);
                        break;
                }
            }
            return true;
        }
    }

    public interface IDocumentView
    {
        Document document
        {
            get;
        }

        event System.Windows.Forms.FormClosedEventHandler FormClosed;

        bool AddElement(Person person);
        bool DeleteElement(Person person);
        bool ModifyElement(Person person);
    }
}
