using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

namespace net1
{
    public partial class TreeViewForm : Form, IDocumentView
    {
        private TreeViewForm()
        {
            InitializeComponent();
        }

        public TreeViewForm(Document doc, System.Windows.Forms.Form mdiParent) : this()
        {
            innerDoc = doc;
            MdiParent = mdiParent;
            doc.Register(this);

            foreach (Person person in doc.peopleList)
            {
                TreeNode item = PersonToTreeNode(person);
                this.treeView1.Nodes.Add(item);
            }

            this.Text = doc.docName;

            Show();
        }

        private TreeNode PersonToTreeNode(Person person)
        {
            TreeNode item = new TreeNode(
            person.personName,
            new TreeNode[] {
                    new TreeNode(person.GetHeightText()),
                    new TreeNode(person.GetWeightText()),
                    new TreeNode(person.GetEyesText())});
            item.Tag = person;
            return item;
        }

        Document innerDoc;

        public Document document
        {
            get { return innerDoc; }
        }

        public bool AddElement(Person person) {
            TreeNode item = PersonToTreeNode(person);
            this.treeView1.Nodes.Add(item);
            return true;
        }

        public bool DeleteElement(Person person) {
            foreach (TreeNode item in this.treeView1.Nodes)
            {
                if (item.Tag == person)
                {
                    this.treeView1.Nodes.Remove(item);
                    return true;
                }
            }
            return false;
        }

        public bool ModifyElement(Person person)
        {
            foreach (TreeNode item in this.treeView1.Nodes)
            {
                if (item.Tag == person)
                {
                    item.Text = person.personName;
                    item.Nodes[0].Text = person.GetHeightText();
                    item.Nodes[1].Text = person.GetWeightText();
                    item.Nodes[2].Text = person.GetEyesText();
                    return true;
                }
            }
            return false;
        }

        private void TreeViewForm_FormClosing(object sender, FormClosingEventArgs e)
        {
            if (e.CloseReason != CloseReason.MdiFormClosing && document.viewCount <= 1)
                e.Cancel = true;
        }

        private void dodajToolStripMenuItem_Click(object sender, EventArgs e)
        {
            document.AddElement();
        }

        private void usuToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (this.treeView1.SelectedNode != null && this.treeView1.SelectedNode.Parent != null)
                document.DeleteElement((Person)this.treeView1.SelectedNode.Parent.Tag);
            else
                document.DeleteElement((Person)this.treeView1.SelectedNode.Tag);
        }

        private void modyfikujToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (this.treeView1.SelectedNode != null && this.treeView1.SelectedNode.Parent != null)
                document.ModifyElement((Person)this.treeView1.SelectedNode.Parent.Tag);
            else
                document.ModifyElement((Person)this.treeView1.SelectedNode.Tag);
        }
    }
}
